#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 05-Mar-2004
################################################################################
#
# Portal - Script part of Ethereal Realms designed to authenthicate and and 
#          lead them to realms
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use eRealms::Audit;						# Auditing handle
use eRealms::Database;						# Database handler
use eRealms::Login;						# Login functionality
use eRealms::Param;						# Parameter control
use eRealms::Rotate;						# Code rotation
use eRealms::Table;						# Table handler
use eRealms::Template;						# Template handler
use eRealms::Weather;						# Weather handler
use eRealms::Who;						# Who is on


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash
my $tmpl;							# Template handle

my $scriptname;							# Simply the script name
my $scriptpara;							# Scripted parameters
my $puppet;							# Puppet name

my %portal;							# Portal hash

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling
	$param    = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Link with hash
	$database->GetHashPortal(\%portal);

	# Initial hash
	$tmpl  = new eRealms::Template(\%portal);

	# Pull script name
	$scriptname = $cgi->url(-relative=>1);

	# Round about way
	if ((length($scriptname) < 2)
	 || ($scriptname =~ /\.(shtml|html)$/i))
	{
		# Assume this is not a real script name
		# Get name and only end
		$scriptname = $0;
		$scriptname =~ s/.*\/(.*)$/$1/gs;
	}

	# Regular operation
	unless ($scriptname =~ /^static/si)
	{
		# Document header
		print $cgi->header();
		$database->DocumentGetHeader();

		# Login and template instance
		$login = new eRealms::Login($database, $cgi, $param);

		# Authenthication
		if ($login->GetVerificationPortal())
		{
			# Quick launch
			if ($scriptname =~ /^go/si)
			{
				# Pull script name
				$scriptname = $cgi->url(-full=>1);
				$scriptpara = $cgi->url(-path_info=>1);

				# Determine need
				# Escape
				$scriptname = quotemeta($scriptname);

				# Truncate
				$scriptpara =~ s/^$scriptname\///;
				$scriptpara =~ s/\/$//;


				# Warp config
				# Need exists
				if (length($scriptpara) > 3)
				{
					# Unescape
					$scriptpara = $cgi->unescape($scriptpara);

					# Parameter assignment
					$param->{'ROOM'} = $scriptpara;
					$param->{'THIRDCHAPTER'} = 'True';
				}
			}
	
			# Otherwise open gateway
			if (defined($param->{'THIRDCHAPTER'}))
			{ 
				# Verification
				# Puppet is defined?
				($puppet) = $database->DataGetPuppetExistence($param->{'CHAR'}, $param->{'USER'});

				# Look for default
				($puppet) = $database->DataGetDefault($param->{'USER'}) unless (defined($puppet)); 


				# Gateway to realms
				if (defined($puppet))
				{
					# Safety first
					$param->{'CHAR'} = $puppet;

					# List realm splash and weather
					ThirdChapter($database, $cgi, $param, $tmpl,  \%portal); 
				}

				else
				{
					# Cleanup
					$param->Cleanup('THIRDCHAPTER');

					# List realms, puppets and links
					SecondChapter($database, $cgi, $param, $tmpl, \%portal);
				}
			}

			# Portal is default
			else
			{
				# List realms, puppets and links
				SecondChapter($database, $cgi, $param, $tmpl, \%portal);
			}
		}

		# Document footer
		$database->DocumentGetFooter();
	}

	# Static/Non Interactive/SSI handler
	else
	{
		# Pointless but needed
		print "<BR>\n\n";

		# Call for operation
		DescriptionStatic($database, $cgi, $tmpl, \%portal);
	}

################################################################################
# Sub-Routines
################################################################################

#####################
# Collect Forum
#
# Will generate a list of all system forums and the amount of entires within
# as to guide people and generate interest.

sub CollectForum
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameters
	my $tmpl     = shift;					# Template handler
	my $system   = shift;					# System parameters
	my $portal   = shift;					# Portal hash

	my $statement;						# Database statement
	my $res;						# Results
	
	my $link;						# Bio link
	my $line;						# Generated line

	#####################
	# Program area
	
	# Initiate line
	$line = '';
	
	# Generate only if necessary
	if ($system->{'SetPortalForum'} =~ /^1/)
	{
		# Prepare query
		$database->Pull(\$statement, "SELECT
			  BoardTopic        AS \"Topic\",
			  COUNT(BoardTopic) AS \"Posts\"
			FROM Board 
			WHERE RealmName=?
			GROUP BY BoardTopic
			ORDER BY BoardTopic", 

		  $system->{'SetBoardAlias'}
 		 );

		# Run through and execute
		while ($res = $statement->fetchrow_hashref())
		{
			# Generate Inline
			$link = $param->Crypt(
				USER  => $param->{'USER'},
				CRYPT => $param->{'CRYPT'},
				ROOM  => $system->{'SetBoardAlias'},
				TOPIC => $res->{'Topic'}
			 );

			# Append to line
			$line = $line . $tmpl->Pass('TagForumItem',
				MTOPIC => $res->{'Topic'},
				MPOSTS => $res->{'Posts'},
				LTOPIC => $portal->{'LnkForum'} . $link
			 );
		}
	
		# Finish query
		$statement->finish();
	}

	# Return value
	return $line;
}

#####################
# Collect News
#
# Will generate a list of the last three news items to catch more attention
# from those that are entering exclusively through the portal.

sub CollectNews
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameters
	my $tmpl     = shift;					# Template handler
	my $system   = shift;					# System parameters
	my $portal   = shift;					# Portal hash

	my $statement;						# Database statement
	my $res;						# Results
	
	my $bio;						# Bio link
	my $post;						# Post link
	my $topc;						# Topic link
	my $line;						# Current line


	#####################
	# Program area
	
	# Initiate line
	$line = '';
	
	# Only generate if required
	if ($system->{'SetPortalNews'} =~ /^1/)
	{
		# Prepare query
		$database->Pull(\$statement, "SELECT
			  LogIDNumber AS \"LogIDNumber\",
			  PuppetName  AS \"PuppetName\",
			  LogTitle    AS \"LogTitle\",
			  LogTopic    AS \"LogTopic\",
			  LogPost     AS \"LogPost\",
			  getDate(LogTimestamp) AS \"LogTimestamp\"
			FROM Weblog
			ORDER BY LogIDNumber DESC
			LIMIT 3");

		# Run through and execute
		while ($res = $statement->fetchrow_hashref())
		{
			# Biographical link
			$bio  = $portal->{'LnkBio'}  . $param->Crypt(BCHAR => $res->{'PuppetName'}, BPOOL => 'system');
			$post = $portal->{'LnkNews'} . $param->Crypt(WID   => $res->{'LogIDNumber'});
			$topc = $portal->{'LnkNews'} . $param->Crypt(FIL   => uc($res->{'LogTopic'}));
	
		
			# Generate line
			$line = $line . $tmpl->Pass('TmplNewsItem',
				MLOGID => $res->{'LogIDNumber'},
				MNAME  => $res->{'PuppetName'},
		 		MTITLE => $res->{'LogTitle'},
				MTOPIC => $res->{'LogTopic'},
				MDATE  => $res->{'LogTimestamp'},
				LBIO   => $bio,
				LPOST  => $post,
				LTOPC  => $topc
			 );
		}

		# Finish query
		$statement->finish();
	}

	# Return value
	return $line;
}


#####################
# Collect Puppets
#
# Will generate the lists for both puppets and change behaviour
# based on what has been pulled if the need for a widget is desired over
# a list based format.

sub CollectPuppets
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameters
	my $tmpl     = shift;					# Template handler
	my $system   = shift;					# System hash
	my $portal   = shift;					# Portal hash
	my $def      = shift;					# Default puppet

	my $line;						# Current line
	my $puppet;						# Singular puppet
	my $widget;						# Generated widget

	my @puppets;						# List of puppets


	#####################
	# Program area

	# Safety catch
	if (defined($def))
	{
		# Fetch list of puppets for system
		$database->GroupList(\@puppets, 'restricted', $param->{'USER'}, 'system');


		# Formatting of options
		# Legacy generation
		if ($system->{'SetPortalPuppet'} =~ /^0/)
		{
			# Generate legacy widget
			$line = $cgi->scrolling_list(
			  -name    => 'CHAR',
			  -values  => \@puppets,
			  -default => $def,
			  -size    => 10,
			  -class   => 'CHAR'
			 );
		}

		# List formatting
		if ($system->{'SetPortalPuppet'} =~ /^1/)
		{
			# Initial formatting
			$line = '';
		
			# Loop and handle appropriatly
			foreach $puppet (@puppets)
			{
				# Nothing to do
				next if ($puppet eq '');

				# Title
				if ($puppet =~ /^--.+--$/)
				{
					# Insert title widget
					$line = $line . $tmpl->Pass('TagPuppetTitle',
						MNAME => $puppet
					 );

				}

				# Handle names
				else
				{
					# generate widget
					$widget = ($puppet eq $def)
						? "<INPUT TYPE=\"radio\" NAME=\"CHAR\" VALUE=\"$puppet\" CHECKED \/>"
						: "<INPUT TYPE=\"radio\" NAME=\"CHAR\" VALUE=\"$puppet\" \/>";
					
					# Create line
					$line = $line . $tmpl->Pass('TagPuppetItem',
						MNAME => $puppet,
						WSLCT => $widget
					 );
						
				}
			}
		}
	}

	# Ensure definition
	$line = (defined($line)) ? $line : '';

	# Return value
	return $line;
}


#####################
# Collect Realms
#
# Will generate the lists for both realms and change behaviour
# based on what has been pulled if the need for a widget is desired over
# a list based format.

sub CollectRealms
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameters
	my $tmpl     = shift;					# Template handler
	my $system   = shift;					# System hash
	my $portal   = shift;					# Portal hash
	my $ver      = shift;					# Verified
	my $js       = shift;					# Javascript aware
	my $def      = shift;					# Default realms

	my $statement;						# Database statement
	my $res;						# Database results
	my $who;						# Who is on handle
	
	my $line;						# Current line
	my $mangle;						# Mangled name

	my @realms;						# Realm public list
	my @ver;						# Verified realms

	my %realms;						# Hash of realms


	#####################
	# Program area

	# Create instance
	$who    = new eRealms::Who($database, $cgi, $param);

	# List of all publis crealms
	$database->GetListPublicRealms(\@realms);	

	# Verified realms
	if ($ver eq 'yes')
	{
		# Pull list
		$database->GetListVerifiedRealms(\@ver);

		# Consolidate, consolidate, consolidate
		@realms = sort(@realms, @ver);
	}

	
	# Final mangling
	$mangle = $who->Portal($def, \@realms, \%realms);


	# Legacy widget formatting
	# Widget Formatting
	if ($system->{'SetPortalRealm'} =~ /^0/)
	{
		# Javascript aware
		if ($js eq 'yes')
		{
			# Generate widget
			$line = $cgi->scrolling_list(
			  -name     => 'ROOM',
			  -values   => \@realms,
			  -default  => $mangle,
			  -size     => 10,
			  -class    => 'ROOM',
			  -onchange => 'createContent()'
			 );
		}

		# Ignorant of javascript
		# To avoid any error messages
		else
		{
			# Generate widget
			$line = $cgi->scrolling_list(
			  -name     => 'ROOM',
			  -values   => \@realms,
			  -default  => $mangle,
			  -size     => 10,
			  -class    => 'ROOM'
			 );
		}
	}

	# List formatting
	if ($system->{'SetPortalRealm'} =~ /^1/)
	{
		# Additional vars
		my $count;					# Counter
		my $homepage;					# Homepage icon
		my $verified;					# Verified realm
		my $widget;					# Widget handler
		
		# Initial formatting
		$line  = '';


		# Cycle through realms
		# Prepare and execute
		$database->Pull(\$statement, "SELECT
			 RealmName        AS \"RealmName\",
			 RealmRating      AS \"RealmRating\",
			 RealmDescription AS \"RealmDescription\",
			 RealmGenre       AS \"RealmGenre\",
			 RealmHomepage    AS \"RealmHomepage\",
			 RealmVerified    AS \"RealmVerified\"
			FROM Realm
			WHERE RealmPublic=?
			   OR RealmVerified=?
			ORDER BY RealmName", 'yes', 'yes');

		# Initialize
		$count = 0;
		
		# Loop and generate lines
		while ($res = $statement->fetchrow_hashref())
		{
			# Check for validity
			if ($realms[$count] =~ /^$res->{'RealmName'}/)
			{
				# Define variable macros
				# Verified realms
				$verified = ($res->{'RealmVerified'} eq 'yes')
			       		? $portal->{'TagRealmVerified'}
				       	: $portal->{'TagRealmPublic'}; 

				# Homepage available
				$homepage = ($res->{'RealmHomepage'} =~ /^http:\/\/.+/i)
			       		? $tmpl->Pass('TagRealmHomeExists', MLINK=>$res->{'RealmHomepage'})
					: $portal->{'TagRealmHomeMissing'};

				# Generate widget
				$widget = ($res->{'RealmName'} eq $def)
					? "<INPUT TYPE=\"radio\" NAME=\"ROOM\" VALUE=\"$res->{RealmName}\" CHECKED \/>"
					: "<INPUT TYPE=\"radio\" NAME=\"ROOM\" VALUE=\"$res->{RealmName}\" \/>";


				# Search and replace
				$line = $line . $tmpl->Pass('TmplRealmItem',
					MDESC => $res->{'RealmDescription'},
					MGENR => $res->{'RealmGenre'},
					MNAME => $res->{'RealmName'},
					MRATE => $res->{'RealmRating'},
					MHOME => $homepage,
					MVERI => $verified,
					MPERS => $realms{$res->{'RealmName'}},
					WSLCT => $widget
				 );
				 
				# Increment
				$count++;
			}
		}

		# Finish statement
		$statement->finish();
	}

	# Ensure definition
	$line = (defined($line)) ? $line : '';

	# Return line
	return $line;
}


	
#####################
# Description Handler
#
# If javascript is enabled, this function will be called.   Once that is done
# the necessary components will be filled in as to allow for use of the 
# javascript descriptions/

sub DescriptionHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $system   = shift;					# System hash
	my $js       = shift;					# Javascript aware


	#####################
	# Program Area

	# Only if necessary
	if (($js = 'yes')
	 && ($system->{'SetPortalRealm'} =~ /^0/))
	{
		#####################
		# Local data members
		my $res;					# Query results
		my $statement;					# Query statements

		my $macro ='';					# Generate macro replacement
		my $descr;					# Description


		# Retreive results
		# Description
		$descr = $database->DocumentGetDescription();


		# Realms and descriptions
		$database->Pull(\$statement, "SELECT
			 RealmName        AS \"RealmName\",
			 RealmDescription AS \"RealmDescription\",
			 RealmGenre       AS \"RealmGenre\",
			 RealmRating      AS \"RealmRating\"
			FROM Realm
			WHERE (RealmPublic='yes'
			   OR RealmVerified='yes')
			  AND RealmDescription IS NOT NULL");


		# Loop and generate
		while ($res = $statement->fetchrow_hashref())
		{
			# Append to macro
			$macro = "$macro\tcase \"$res->{RealmName}\":\n\t" 
			 . " mes='$res->{RealmDescription}'\;\n\t"
			 . " rat='$res->{RealmRating}'\;\n\t"
			 . " gre='$res->{RealmGenre}'\;\n\t"
			 . " break\;\n";
		}


		# Complete query
		$statement->finish();

		
		# Replace macro
		$descr =~ s/MDESCRCASE/$macro/gs;

		# Display
		print "$descr\n\n";
	}
}


#####################
# Description Static
#
# A variant of the above with purpose of displaying this to mostly
# static pages.  Meaning being used in Server Side Includes (SSI)
# et cetera.

sub DescriptionStatic
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common Gateway Interface
	my $tmpl     = shift;					# Template handler
	my $portal   = shift;					# Portal handler

	my $res;						# Query results
	my $statement;						# Query statements
	my $table;						# Table handler	

	my $escape;						# Escaped title
	my $genre;						# Current genre
	my $title;						# Displayed title
	my $url;						# Quick access link

	my @genres;						# List of genres


	#####################
	# Program Area

	# Define instance
	$table = new eRealms::Table($cgi);


	# Genres
	# Pull values
	$database->GetList(\@genres, "SELECT RealmGenre
		FROM Realm
		WHERE (RealmPublic='yes'
		   OR RealmVerified='yes')
		GROUP BY RealmGenre
		ORDER BY RealmGenre");

	# Push
	push(@genres, $portal->{'TxtGlobal'});


	# Generate nav links
	QuickLink($cgi, $tmpl, \@genres);


	# Clear value
	$genre = $genres[0];


	# Realm specific
	# Retreive results
	# Realms and descriptions
	$database->Pull(\$statement, "SELECT
		 RealmName         AS \"RealmName\",
		 RealmHomepage     AS \"RealmHomepage\",
		 RealmDescription  AS \"RealmDescription\",
		 RealmGenre        AS \"RealmGenre\",
		 RealmRating       AS \"RealmRating\"
		FROM Realm
		WHERE RealmPublic <> 'no'
		  AND RealmDescription IS NOT NULL
		  AND RealmHomepage IS NOT NULL
		ORDER BY RealmGenre, RealmName");
	$statement->execute();



	#  Title for first genre
	# Escape
	$escape = $cgi->escape($genre);

	# Title
	$tmpl->Show('TmplGenre', MTITLE=>$genre, MANCHOR=>"NAME=\"$escape\"");

	# Table top
	$table->MakeTop("95%");

	# Loop and generate
	while ($res = $statement->fetchrow_hashref())
	{
	 	# Genre handling	
		if ($genre ne $res->{'RealmGenre'})
		{
			# Assign and escape
			$genre  = $res->{'RealmGenre'};
			$escape = $cgi->escape($genre);

			# Spacer
			$table->MakeBottom();


			# Quick nav
			QuickLink($cgi, $tmpl, \@genres, $genre);


			# Title
			$tmpl->Show('TmplGenre',
 			 MTITLE  => $genre,
			 MANCHOR => "NAME=\"$escape\""
			);


			# Start anew 
			$table->MakeTop("95%");
		}


		# Main generation
		# Generate link
		$url = $cgi->escape($res->{'RealmName'});
		$url = $portal->{'LnkQuickLink'} . "/" . $url;

		# Information and spacer
		$table->MakeSingle(
		  $tmpl->Pass('TmplStatic',
		   MHOME   => $res->{'RealmHomepage'},
		   MLINK   => $url,
	 	   MRATING => $res->{'RealmRating'},
	 	   MTITLE  => $res->{'RealmName'}
		  )
		 );
		$table->MakeSingle("<P ALIGN=\"justify\">$res->{RealmDescription}</P>");
		$table->MakeBlank();
	}

	# Table bottom
	$table->MakeBottom();


	# Global list
	# Retreive results
	# Realms and descriptions
	$database->Pull(\$statement, "SELECT
		 RealmName        AS \"RealmName\",
		 RealmHomepage    AS \"RealmHomepage\",
		 RealmDescription AS \"RealmDescription\",
		 RealmRating      AS \"RealmRating\"
		FROM Realm
		WHERE RealmPublic <> 'no' 
		  AND RealmDescription IS NOT NULL
		  AND RealmHomepage IS NOT NULL
		ORDER BY RealmName");


	# Quick nav
	QuickLink($cgi, $tmpl, \@genres, $portal->{'TxtGlobal'});


	#  Title for first genre
	# Escape
	$escape = $cgi->escape($portal->{'TxtGlobal'});

	# Title
	$tmpl->Show('TmplGenre',
 		MTITLE  => $portal->{'TxtGlobal'},
		MANCHOR => "NAME=\"$escape\""
	);

	# Table top
	$table->MakeTop("95%");


	# Loop and generate
	while ($res = $statement->fetchrow_hashref())
	{
		# Main generation
		# Generate link
		$url = $cgi->escape($res->{'RealmName'});
		$url = $portal->{'LnkQuickLink'} . "/" . $url;

		# Information and spacer
		$table->MakeSingle(
		  $tmpl->Pass('TmplStatic',
		   MHOME   => $res->{'RealmHomepage'},
		   MLINK   => $url,
	 	   MRATING => $res->{'RealmRating'},
	 	   MTITLE  => $res->{'RealmName'}
		  )
		 );
		$table->MakeSingle("<P ALIGN=\"justify\">$res->{RealmDescription}</P>");
		$table->MakeBlank();
	}

	# Table bottom
	$table->MakeBottom();


	# Generate nav links
	QuickLink($cgi, $tmpl, \@genres);
}


#####################
# Quick Link
#
# Dirt simple and is used simply to clean up the static pages list of elements
# allowing for quick navigation.

sub QuickLink
{
	#####################
	# Data members
	my $cgi     = shift;					# CGI handle
	my $tmpl    = shift;					# Template handle
	my $array   = shift;					# Referenced array
	my $exclude = shift;					# Exclusion

	my $escape;						# Escaped name
	my $genre;						# Genre


	#####################
	# Program area

	# Pre-Display
	print "<CENTER>";

	# Loop and display
	foreach $genre (@{$array})
	{
		unless ((defined($exclude))
		 && ($genre eq $exclude))
		{
			# Escape link
			$escape = $cgi->escape($genre);

			# Simply display from a format
			$tmpl->Show('TagLnkGen', MTITLE=>$genre, MLINK=>"HREF=\"\#$escape\"");
		}
	}

	# Post
	print "</CENTER>\n\n";

	# Spacer
	print "<BR>\n";
}


#####################
# Second Chapter
#
# Unusual name given to a part of the script that will be exected once the user
# has been authenthicated.   

sub SecondChapter
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $portal   = shift;					# Portal hash

	my $rotate;						# Code rotation

	my $def_pup;						# Default puppet
	my $def_realm;						# Default realm
	my $key;						# Singular key
	my $mail;						# Mail count
	my $news;						# News entries
	my $new;						# Newness factor
	my $template;						# Actual template

	my $sup;						# Supervisory
	my $adm;						# Administrative
	my $mod;						# Moderators
	my $ver;						# Verified users

	my $js;							# Javascript control
	my $jscode;						# Javascript code

	my %keys;						# Generated keys
	my %system;						# System level parameters

	#####################
	# Program area

	# Link with hash
	$database->GetHashSystem(\%system);


	# Create instances
	$rotate = new eRealms::Rotate(DB=>$database, ZONE=>'portal');


	# Javascript Options
	# Retreive Javascript choice
	($js) = $database->DataGetJavascript($param->{'USER'});

	# Generate javascript code
	$keys{'GENJSCODE'} = ($js ne 'no') ? "ONCLICK='window.open(this.href,this.target,settings); return false'" : '';


	# Retreive puppet default
	($def_pup, $def_realm) = $database->DataGetDefault($param->{'USER'}); 

	# Retreive numbers
	($mail) = $database->DataGetPuppeteerMailCount($param->{'USER'});
	($news, $new) = $database->DataGetWeblogEntries();

	# Determine priviliges
	($sup, $adm, $mod, $ver) = $database->DataGetPrivs($param->{'USER'});


	# Set general macros
	# Counters
	$keys{'MMAILCOUNT'}  = $mail;
	$keys{'MNEWSCOUNT'}  = $news;

	# Format and seting of recent
	$keys{'MNEWSRECENT'} = ($new != 0) ? $portal->{'TagNewness'} : '';

	# Possibly display Javascript
	$keys{'MJAVASCRIPT'} = ($js eq 'yes') ? $database->DocumentGetJavascript() : ''; 

	# Create private realm
	$keys{'WPRIVREALM'}  = $cgi->textfield(
		-name      => 'ALTROOM', 
		-default   => '', 
		-size      => 20,
	        -maxlength => 30,
		-class     => 'ALTROOM');

	# Submit widget
	$keys{'WSUBMIT'}     = $cgi->submit(
		-name      => $portal->{'TxtSubmitRealms'},
	        -value     => 'submit',
	        -class     => 'WSUBMIT');

	# Create inline link
	$keys{'GENPARAM'}    = $param->EmbedInline(
		USER  => $param->{'USER'},
		CRYPT => $param->{'CRYPT'}
	 );


	# Lists
	# Listing of all realms
	$keys{'LSTREALMS'}   = CollectRealms($database, $cgi, $param, $tmpl, \%system, $portal, $ver, $js, $def_realm);

	# Listing of all puppets
	$keys{'LSTPUPPETS'}  = CollectPuppets($database, $cgi, $param, $tmpl, \%system, $portal, $def_pup);

	# Listing of all news entries
	$keys{'LSTNEWS'}     = CollectNews($database, $cgi, $param, $tmpl, \%system, $portal);

	# Listing for all forum pieces
	$keys{'LSTFORUM'}    = CollectForum($database, $cgi, $param, $tmpl, \%system, $portal);


	# Sections
	# Rotated section
	$keys{'MROTSECT'} = $rotate->Pull();

	# Pre-generation for most of population
	$keys{'MADMSECT'} = '';
	$keys{'MSUPSECT'} = '';


	# Generate specific sections
	# Supervisory and Moderator
	if (($sup eq 'yes')
	 or ($mod eq 'yes'))
	{
		# Complete template
		$keys{'MSUPSECT'} = $tmpl->Pass('TmplSuperSect',
			GENPARAM  => $keys{'GENPARAM'},
			GENJSCODE => $keys{'GENJSCODE'}
		 );
	}

	# Administrative
	if ($adm eq 'yes')
	{
		# Complete template
		$keys{'MSUPSECT'} = $tmpl->Pass('TmplAdminSect',
			GENPARAM  => $keys{'GENPARAM'},
			GENJSCODE => $keys{'GENJSCODE'}
		 );
	}
	
	# Fetch template
	($template) = $database->DocumentGetPortal();
	

	# Generation
	# Show descriptions
	DescriptionHandler($database, \%system, $js);


	# Run through keys
	foreach $key (keys(%keys))
	{
		# Replace every occurence
		$template =~ s/$key/$keys{$key}/gs;
	}

	# Start form
	print $cgi->start_form(-NAME=>'PORTAL');
	print $param->EmbedNormal($param->Flat(), THIRDCHAPTER=>'True'), "\n";

	# Final display
	print $template, "\n";
	
	# Close form
	print $cgi->end_form(), "\n";
}


#####################
# Third Chapter
#
# Unusual name given to a part of the script that will be exected once the user  
# in his infinit wisdom has selected a puppet and realm

sub ThirdChapter
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template handler
	my $portal   = shift;					# Portal hash

	my $who;						# Who is on handle

	my $access;						# Access control
	my $alias;						# Room alias
	my $room;						# Room name
	my $info;						# Room information
	my $pub;						# Public status
	my $rand;						# Random password
	my $status;						# Room status
	my $ver;						# Verified requirement


	#####################
	# Program area

	# Cleanup
	delete $param->{'THIRDCHAPTER'};


	# Determine need to create a room
	if ((defined($param->{'ALTROOM'}))
	 && (length($param->{'ALTROOM'}) > 2))
	{
		# Existence of password
		if ($param->{'ALTROOM'} =~ /@/)
		{
			# Seperate into password and realm
			($param->{'RPASS'}, $param->{'ALTROOM'}) = split(/@/, $param->{'ALTROOM'});

			# Meets length requirements
			$param->{'ROOM'} = (length(($param->{'ALTROOM'})) > 2) ? $param->{'ALTROOM'} : $param->{'ROOM'};
		}

		# Default
		else
		{	
			# Assign ROOM
			$param->{'ROOM'} = $param->{'ALTROOM'};
		}

		# Retreive query information
		($room, $status, $alias) = $database->DataGetRealmExistence($param->{'ROOM'});


		# Create if not found
		unless (defined($room))
		{
			# Auditing instance
			my $audit = new eRealms::Audit($database, $cgi, $param);


			# Handle realm password
			if ((defined($param->{'RPASS'})) 
			 && (length($param->{'RPASS'}) > 2))
			{
				# Generate password
				$rand  = md5_hex(rand());
				$rand  = substr($rand, 0, 14);

				# Generated encrypted password
				$param->{'RCRYPT'} = crypt($param->{'RPASS'}, $rand);

				# Cleanup
				$param->Cleanup('RPASS');
			}
			else
			{
				# Undefine parameter
				$param->{'RCRYPT'} = undef;
			}

			# Evaluate block
			eval
			{
				# Realm creation
				$database->Write("INSERT INTO Realm 
					(RealmName,
					 RealmPassword,
					 RealmImageHeight,
					 RealmImageWidth,
					 RealmImageSize)
					VALUES(?,?,0,0,0)", 

				 $param->{'ROOM'}, 
				 $param->{'RCRYPT'} 
				);

				# Board link
				$database->Write("INSERT INTO BoardTopic
					(RealmName,
					 BoardTopic)
					VALUES (?,?)",

				 $param->{'ROOM'},
				 $portal->{'TxtDefault'}
				);

				# Record addition
				$audit->UserRealm($param->{'ROOM'}, $param->{'ROOM'});


				# Commit changes
				$database->Commit();
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();


				# Re-display portal
				SecondChapter($database, $cgi, $param, $tmpl, $portal);

			 	# Return
				return 0;	
			}
		}
		

		# Realm exists
		else
		{
			# Handle alias
			$room = ($status eq 'alias')  ? $alias : $room;

			# Consistent naming
			$param->{'ROOM'} = $room;
		}
	}

	# Remove number information
	$param->{'ROOM'} =~ s/ \(\d+\)$//s;


	# Access control
	($access) = $database->DataGetRealmAccess($param->{'ROOM'});

	# Ensure puppet is defined
	if (($access eq 'restricted')
	 or (!defined($param->{'CHAR'})))
	{
		# Retreive default puppet
		($param->{'CHAR'}) = $database->DataGetDefault($param->{'USER'}, $param->{'ROOM'}, $access);
	}

	# Retreive realm information
	($info)      = $database->DataGetRealmDescription($param->{'ROOM'});
	($pub, $ver) = $database->DataGetRealmPublic($param->{'ROOM'});


	# Update default realm
	if (($pub eq 'yes')
         || ($ver eq 'yes'))
	{
		# Update default realm if public or adult
		$database->Quick("UPDATE Puppeteer
			SET PuppeteerRealm=?
			WHERE PuppeteerLogin=?",

		 $param->{'ROOM'},
		 $param->{'USER'}
		);
	}

	
	# who is on handles
	$who = new eRealms::Who($database, $cgi, $param);
	
	# Wrap with tables
	if (defined($info))
	{	
		# Wrap with tables
		print "<TABLE WIDTH=\"100%\">\n",
		      " <TR>\n",
		      "  <TD VALIGN=\"top\">";

		# Retreive realm information
		$tmpl->ShowSub($param->{'ROOM'});
		$who->WhoEntrance();

		# Warn before entering
		$database->DocumentGetCanon() if ($access eq 'restricted');

		# Weather forcast
		WeatherHandler($database, $cgi, $param, $tmpl, $portal);

		# Mid section
		print "  </TD>\n",
		      "  <TD VALIGN=\"top\">";
		
		# Splash page
		print "$info\n";

		# Hind quarter
		print "\n  </TD>\n",
		      " </TR>\n",
		      "</TABLE>";
	}

	# Simply display
	else
	{
		# Retreive realm information
		$tmpl->ShowSub($param->{'ROOM'});
		$who->WhoEntrance();

		# Warn before entering
		$database->DocumentGetCanon() if ($access eq 'restricted');

		# Weather forcast
		WeatherHandler($database, $cgi, $param, $portal);
	}
}


#####################
# Weather Handler
#
# The weather heandler is realm specific for which is it not listed on the
# module directly.   This will essentially show off the specific weather 
# patterns for the realm if found

sub WeatherHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template
	my $portal   = shift;					# Portal handle

	my $table;						# Table handle

	my $for;						# Forcast handle
	my $metar;						# Coded weather forcast

	my $cond;						# Weather conditions
	my $sky;						# Sky conditions

	my %cond;						# Specific conditions


	#####################
	# Program area

	# Create instance
	$table    = new eRealms::Table($cgi);

	# Retreive forcast
	($metar) = $database->DataGetWeather($param->{'ROOM'});
	

	# Only complete if forcast
	if (defined($metar))
	{
		# Initialize handle
		$for = new eRealms::Weather();

		# Parse
		$for->Parse($metar, \%cond);


		# Forcast handling
		$sky  = lc(join(", ", @{$cond{SKY}}));
		$cond = lc(join(", ", @{$cond{HUMAN}}));		

		# Conditions
		$cond = (length($cond) > 2) ? ucfirst("$cond, $sky") : ucfirst("$sky");
		$cond =~ s/,([^,.]+?)$/ and $1/s;


		# Title
		print "<BR>\n";
		$tmpl->ShowSub($portal->{'TagForcastTitle'});

		# Display area
		$table->MakeTop("100%");
		$table->MakeSingle("$portal->{TagForcastCond}<BR>$cond");

		# Wind
		if ($cond{WIND_MPH} != 0)
		{ $table->MakeSingle("$portal->{TagForcastWind}<BR>$cond{WIND_DIR_ENG} at $cond{WIND_KPH} KPH - $cond{WIND_MPH} MPH"); }

		# Temperature
		if ($cond{F_TEMP} != 0)
		{ $table->MakeSingle("$portal->{TagForcastTemp}<BR>$cond{C_TEMP}C - $cond{F_TEMP}F" ); }

		# Dew points
		if ($cond{F_DEW} != 0)
		{ $table->MakeSingle("$portal->{TagForcastDew} <BR>$cond{C_DEW}C - $cond{F_DEW}F"); }

		# Visibility
		if ($cond{VIS_MILE} != 0)
		{ $table->MakeSingle("$portal->{TagForcastVis} <BR>$cond{VIS_KILO} Km - $cond{VIS_MILE} Mi"); }

		# Barometer
		if ($cond{ALTI} != 0)
		{ $table->MakeSingle("$portal->{TagForcastBaro}<BR>$cond{ALTI} Inches - $cond{ALTK} Kilopascals"); }

		# Table bottom
		$table->MakeBottom();
	}
}
